#*****************************************************************************
#* File: lua/modules/ui/game/orders.lua
#* Author: Chris Blackwell
#* Summary: Unit orders UI
#*
#* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Grid = import('/lua/maui/grid.lua').Grid
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local Button = import('/lua/maui/button.lua').Button
local Tooltip = import('/lua/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/ui/help/tooltips.lua')
local Prefs = import('/lua/user/prefs.lua')
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local CM = import('/lua/ui/game/commandmode.lua')

controls =
{
    mouseoverDisplay = false,
    orderButtonGrid = false,
    bg = false,
    orderGlow = false,
    controlClusterGroup = false,
    mfdControl = false,
}

# positioning controls, don't belong to file
local layoutVar = false
local glowThread = false

# these variables control the number of slots available for orders
# though they are fixed, the code is written so they could easily be made soft
local numSlots = 12
local firstAltSlot = 7
local vertRows = 3
local horzRows = 4
local vertCols = numSlots/vertRows
local horzCols = numSlots/horzRows

local function CreateOrderGlow(parent)
    controls.orderGlow = Bitmap(parent, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    LayoutHelpers.AtCenterIn(controls.orderGlow, parent)
    controls.orderGlow:SetAlpha(0.0)
    controls.orderGlow:DisableHitTest()
    controls.orderGlow:SetNeedsFrameUpdate(true)
    local alpha = 0.0
    local incriment = true
    controls.orderGlow.OnFrame = function(self, deltaTime)
        if incriment then
            alpha = alpha + (deltaTime * 1.2)
        else
            alpha = alpha - (deltaTime * 1.2)
        end
        if alpha < 0 then
            alpha = 0.0
            incriment = true
        end
        if alpha > .4 then
            alpha = .4
            incriment = false
        end
        controls.orderGlow:SetAlpha(alpha)
    end     
end

local function CreateAutoBuildEffect(parent)
    local glow = Bitmap(parent, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    LayoutHelpers.AtCenterIn(glow, parent)
    glow:SetAlpha(0.0)
    glow:DisableHitTest()
    glow:SetNeedsFrameUpdate(true)
    glow.alpha = 0.0
    glow.incriment = true
    glow.OnFrame = function(self, deltaTime)
        if self.incriment then
            self.alpha = self.alpha + (deltaTime * .35)
        else
            self.alpha = self.alpha - (deltaTime * .35)
        end
        if self.alpha < 0 then
            self.alpha = 0.0
            self.incriment = true
        end
        if self.alpha > .4 then
            self.alpha = .4
            self.incriment = false
        end
        self:SetAlpha(self.alpha)
    end   
    return glow  
end

function CreateMouseoverDisplay(parent, ID)
    if controls.mouseoverDisplay then
        controls.mouseoverDisplay:Destroy()
        controls.mouseoverDisplay = false
    end
    
    if not Prefs.GetOption('tooltips') then return end
    
    local createDelay = Prefs.GetOption('tooltip_delay') or 0
    
    local text = TooltipInfo['Tooltips'][ID]['title'] or ID
    local desc = TooltipInfo['Tooltips'][ID]['description'] or ID
    
    if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
        for i, v in Keymapping do
            if v.stringkey == TooltipInfo['Tooltips'][ID]['keyID'] then
                text = LOCF("%s (%s)", text, i)
                break
            end
        end
    end
    
    if not text or not desc then
        return
    end

    controls.mouseoverDisplay = Tooltip.CreateExtendedToolTip(parent, text, desc)
    local Frame = GetFrame(0)
    controls.mouseoverDisplay.Bottom:Set(parent.Top)
    if (parent.Left() + (parent.Width() / 2)) - (controls.mouseoverDisplay.Width() / 2) < 0 then
        controls.mouseoverDisplay.Left:Set(4)
    elseif (parent.Right() - (parent.Width() / 2)) + (controls.mouseoverDisplay.Width() / 2) > Frame.Right() then
        controls.mouseoverDisplay.Right:Set(function() return Frame.Right() - 4 end)
    else
        LayoutHelpers.AtHorizontalCenterIn(controls.mouseoverDisplay, parent)
    end
    
    local alpha = 0.0
    controls.mouseoverDisplay:SetAlpha(alpha, true)
    local mdThread = ForkThread(function()
        WaitSeconds(createDelay)
        while alpha <= 1.0 do
            controls.mouseoverDisplay:SetAlpha(alpha, true)
            alpha = alpha + 0.1
            WaitSeconds(0.01)
        end
    end)

    controls.mouseoverDisplay.OnDestroy = function(self)
        KillThread(mdThread)  
    end
end

local function CreateOrderButtonGrid()
    controls.orderButtonGrid = Grid(controls.bg, 47, 47)
    controls.orderButtonGrid:SetName("Orders Grid")
	controls.orderButtonGrid:DeleteAll()
end

# local logic data
local orderCheckboxMap = false
local currentSelection = false

# helper function to create order bitmaps
# note, your bitmaps must be in /game/orders/ and have the standard button naming convention
local function GetOrderBitmapNames(bitmapId)
    if bitmapId == nil then
        LOG("Error - nil bitmap passed to GetOrderBitmapNames")
        bitmapId = "basic-empty"    --TODO do I really want to default it?
    end
    
    local button_prefix = "/mods/OllosInterface/textures/icons_orders/" .. bitmapId .. "_btn_"
    return button_prefix .. "up.dds"
        ,  button_prefix .. "up_sel.dds"
        ,  button_prefix .. "over.dds"
        ,  button_prefix .. "over_sel.dds"
        ,  button_prefix .. "dis.dds"
        ,  button_prefix .. "dis_sel.dds"
        , "UI_Action_MouseDown", "UI_Action_Rollover"   -- sets click and rollover cues
end

# used by most orders, which start and stop a command mode, so they toggle on when pressed
# and toggle off when done
local function StandardOrderBehavior(self, modifiers)
    # if we're checked, end the current command mode, otherwise start it
    if self:IsChecked() then
        import('/lua/ui/game/commandmode.lua').EndCommandMode(true)
    else
        import('/lua/ui/game/commandmode.lua').StartCommandMode("order", {name=self._order})
    end
end

# used by orders that happen immediately and don't change the command mode (ie the stop button)
local function DockOrderBehavior(self, modifiers)
    if modifiers.Shift then
        IssueDockCommand(false)
    else
        IssueDockCommand(true)
    end
    self:SetCheck(false)
end

# used by orders that happen immediately and don't change the command mode (ie the stop button)
local function MomentaryOrderBehavior(self, modifiers)
    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    self:SetCheck(false)
end

# used by things that build weapons, etc
local function BuildOrderBehavior(self, modifiers)
    if modifiers.Left then
	    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    elseif modifiers.Right then
	    self:ToggleCheck()
        if self:IsChecked() then
            self._curHelpText = self._data.helpText .. "_auto"
            self.autoBuildEffect = CreateAutoBuildEffect(self)
        else
            self._curHelpText = self._data.helpText
            self.autoBuildEffect:Destroy()
        end
        if controls.mouseoverDisplay.text then
            controls.mouseoverDisplay.text:SetText(self._curHelpText)
        end
	    SetAutoMode(currentSelection, self:IsChecked())
    end
end

local function BuildInitFunction(control, unitList)
    local isAutoMode = GetIsAutoMode(unitList)
    control:SetCheck(isAutoMode)
    if isAutoMode then
        control._curHelpText = control._data.helpText .. "_auto"
        control.autoBuildEffect = CreateAutoBuildEffect(control)
    else
        control._curHelpText = control._data.helpText
    end
end

# used by subs that can dive/surface
local function DiveOrderBehavior(self, modifiers)
    if modifiers.Left then
	    IssueCommand(GetUnitCommandFromCommandCap(self._order))
        self:SetCheck(false)
    elseif modifiers.Right then
	    self:ToggleCheck()
        if self:IsChecked() then
            self._curHelpText = self._data.helpText .. "_auto"
            self.autoBuildEffect = CreateAutoBuildEffect(self)
        else
            self._curHelpText = self._data.helpText
            self.autoBuildEffect:Destroy()
        end
        if controls.mouseoverDisplay.text then
            controls.mouseoverDisplay.text:SetText(self._curHelpText)
        end
	    SetAutoSurfaceMode(currentSelection, self:IsChecked())
    end
end

local function DiveInitFunction(control, unitList)
    local isAutoMode = GetIsAutoSurfaceMode(unitList)
    control:SetCheck(isAutoMode)
    if isAutoMode then
        control._curHelpText = control._data.helpText .. "_auto"
        control.autoBuildEffect = CreateAutoBuildEffect(control)
    else
        control._curHelpText = control._data.helpText
    end
end

# pause button specific behvior
# TODO pause button will be moved to construction manager
local function PauseOrderBehavior(self, modifiers)
	Checkbox.OnClick(self)
	SetPaused(currentSelection, self:IsChecked())
end

local function PauseInitFunction(control, unitList)
    control:SetCheck(GetIsPaused(unitList))
end

function TogglePauseState()
    local pauseState = GetIsPaused(currentSelection)
    SetPaused(currentSelection, not pauseState)
end

# some toggleable abilities need reverse semantics.
local function CheckReverseSemantics(scriptBit)
    if scriptBit == 0 then # shields
        return true
    end
    
    return false
end

local function AbilityButtonBehavior(self, modifiers)
    if self:IsChecked() then
        CM.EndCommandMode(true)
    else
        local modeData = {
            name="RULEUCC_Script", 
            AbilityName=self._script,
            TaskName=self._script,
        }
        CM.StartCommandMode("order", modeData)
    end
end

# generic script button specific behvior
local function ScriptButtonOrderBehavior(self, modifiers)
    local state = self:IsChecked()
    if CheckReverseSemantics(self._data.extraInfo) then
        state = not state
    end
	ToggleScriptBit(currentSelection, self._data.extraInfo, state)
	Checkbox.OnClick(self)
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
end

local function ScriptButtonInitFunction(control, unitList)
    local result = GetScriptBit(unitList, control._data.extraInfo)
    if CheckReverseSemantics(control._data.extraInfo) then
        result = not result
    end
    control:SetCheck(result)
end

local function DroneBehavior(self, modifiers)
    if modifiers.Left then
        SelectUnits( { self._unit } )
    end

    if modifiers.Right then
        if self:IsChecked() then
            self._unit:ProcessInfo('SetAutoMode', 'false')
            self:SetCheck(false)
        else
            self._unit:ProcessInfo('SetAutoMode', 'true')
            self:SetCheck(true)
        end
    end
end

local function DroneInit(self, selection)
    if self._unit:IsAutoMode() then
        self:SetCheck(true)
    else
        self:SetCheck(false)
    end
end

-- retaliate button specific behvior
local retaliateStateInfo = {
    [-1] = {bitmap = 'stand-ground',    helpText = "mode_mixed",},
    [0] = { bitmap = 'return-fire',     helpText = "mode_return_fire",},
    [1] = { bitmap = 'hold-fire',       helpText = "mode_hold_fire",},
    [2] = { bitmap = 'stand-ground',    helpText = "mode_hold_ground",},
}

local function RetaliateOrderBehavior(self, modifiers)
	ToggleFireState(currentSelection, self._toggleState)
	
	self._toggleState = self._toggleState + 1
	if self._toggleState >= 3 then
	    self._toggleState = 0
	end
	
    if not retaliateStateInfo[self._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    self:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[self._toggleState].bitmap))
    self._curHelpText = retaliateStateInfo[self._toggleState].helpText
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
end

local function RetaliateInitFunction(control, unitList)
    control._toggleState = GetFireState(unitList)
    if not retaliateStateInfo[control._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    control:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[control._toggleState].bitmap))
    control._curHelpText = retaliateStateInfo[control._toggleState].helpText
end

function CycleRetaliateStateUp()
    local currentFireState = GetFireState(currentSelection)
    if currentFireState > 3 then
        currentFireState = 0
    end
    ToggleFireState(currentSelection, currentFireState)
end

local function pauseFunc()
    import('/lua/ui/game/queue.lua').EnablePauseToggle()
end

function EnterOverchargeMode()
    local econData = GetEconomyTotals()
    if not currentSelection[1] or currentSelection[1]:IsDead() then return end
    local bp = currentSelection[1]:GetBlueprint()
    local overchargeLevel = 0
    local overchargeFound = false
    for index, weapon in bp.Weapon do
        if weapon.OverChargeWeapon then
            overchargeLevel = weapon.EnergyRequired
            overchargeFound = true
            break
        end
    end
    if overchargeFound then
        if overchargeLevel > 0 and econData["stored"]["ENERGY"] > overchargeLevel then
            ConExecute('StartCommandMode order RULEUCC_Overcharge')
        end
    end
end

local function OverChargeFrame(self, deltaTime)
    if deltaTime then
        local econData = GetEconomyTotals()
        local bp = currentSelection[1]:GetBlueprint()
        local overchargeLevel = 0
        for index, weapon in bp.Weapon do
            if weapon.OverChargeWeapon then
                overchargeLevel = weapon.EnergyRequired
                break
            end
        end
        if overchargeLevel > 0 then
            if econData["stored"]["ENERGY"] > overchargeLevel then
                if self:IsDisabled() then
                    self:Enable()
                    local armyTable = GetArmiesTable()
                    local facStr = import('/lua/factions.lua').Factions[armyTable.armiesTable[armyTable.focusArmy].faction + 1].SoundPrefix
                    local sound = Sound({Bank = string.format('COMPUTER_%s_VO', facStr), Cue = string.format('%sComputer_Basic_Orders_01173', facStr)})
                    PlayVoice(sound)
                end
            else
                if not self:IsDisabled() then
                    self:Disable()
                end
            end
        else
            self:SetNeedsFrameUpdate(false)
        end
    end
end

# sets up an orderInfo for each order that comes in
# preferredSlot is custom data that is used to determine what slot the order occupies
# initialStateFunc is a function that gets called once the control is created and allows you to set the initial state of the button
#      the function should have this declaration: function(checkbox, unitList)
# extraInfo is used for storing any extra information required in setting up the button
local standardOrdersTable = {
    # Common rules
    RULEUCC_Move = {                helpText = "move",          bitmapId = 'move',                  preferredSlot = 1,  behavior = StandardOrderBehavior,},
    RULEUCC_Attack = {              helpText = "attack",        bitmapId = 'attack',                preferredSlot = 2,  behavior = StandardOrderBehavior, },
    RULEUCC_Patrol = {              helpText = "patrol",        bitmapId = 'patrol',                preferredSlot = 3,  behavior = StandardOrderBehavior, },
    RULEUCC_Stop = {                helpText = "stop",          bitmapId = 'stop',                  preferredSlot = 4,  behavior = MomentaryOrderBehavior, },
    RULEUCC_Guard = {               helpText = "assist",        bitmapId = 'guard',                 preferredSlot = 5,  behavior = StandardOrderBehavior, },
    RULEUCC_RetaliateToggle = {     helpText = "mode",          bitmapId = 'stand-ground',          preferredSlot = 6,  behavior = RetaliateOrderBehavior,      initialStateFunc = RetaliateInitFunction, },

    # Unit specific rules
    RULEUCC_SiloBuildTactical = {   helpText = "build_tactical",bitmapId = 'silo-build-tactical',   preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_SiloBuildNuke = {       helpText = "build_nuke",    bitmapId = 'silo-build-nuke',       preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_Overcharge = {          helpText = "overcharge",    bitmapId = 'overcharge',            preferredSlot = 7,  behavior = StandardOrderBehavior,       onframe = OverChargeFrame},
    RULEUCC_Script = {       helpText = "special_action",bitmapId = 'overcharge',                   preferredSlot = 7,  behavior = StandardOrderBehavior,},
    RULEUCC_Transport = {           helpText = "transport",     bitmapId = 'unload',                preferredSlot = 8,  behavior = StandardOrderBehavior, },
    RULEUCC_Nuke = {                helpText = "fire_nuke",     bitmapId = 'launch-nuke',           preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Tactical = {            helpText = "fire_tactical", bitmapId = 'launch-tactical',       preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Teleport = {            helpText = "teleport",      bitmapId = 'teleport',              preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Ferry = {               helpText = "ferry",         bitmapId = 'ferry',                 preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Sacrifice = {           helpText = "sacrifice",     bitmapId = 'sacrifice',             preferredSlot = 9,  behavior = StandardOrderBehavior,},
    RULEUCC_Dive = {                helpText = "dive",          bitmapId = 'dive',                  preferredSlot = 10, behavior = DiveOrderBehavior, initialStateFunc = DiveInitFunction,},   
    RULEUCC_Reclaim = {             helpText = "reclaim",       bitmapId = 'reclaim',               preferredSlot = 10, behavior = StandardOrderBehavior, },
    RULEUCC_Capture = {             helpText = "capture",       bitmapId = 'convert',               preferredSlot = 11, behavior = StandardOrderBehavior,},
    RULEUCC_Repair = {              helpText = "repair",        bitmapId = 'repair',                preferredSlot = 12, behavior = StandardOrderBehavior,},
    RULEUCC_Dock = {                helpText = "dock",          bitmapId = 'repair',                preferredSlot = 12, behavior = DockOrderBehavior,},
    
    DroneL = {              helpText = "drone",        bitmapId = 'unload02',                preferredSlot = 11, behavior = DroneBehavior,initialStateFunc = DroneInit,},
    DroneR = {              helpText = "drone",        bitmapId = 'unload02',                preferredSlot = 11, behavior = DroneBehavior,initialStateFunc = DroneInit,},

    # Unit toggle rules
    RULEUTC_ShieldToggle = {        helpText = "toggle_shield",     bitmapId = 'shield',                preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 0,},
    RULEUTC_WeaponToggle = {        helpText = "toggle_weapon",     bitmapId = 'toggle-weapon-direct',  preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 1,},    
    RULEUTC_JammingToggle = {       helpText = "toggle_jamming",    bitmapId = 'jamming',               preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 2,},
    RULEUTC_IntelToggle = {         helpText = "toggle_intel",      bitmapId = 'intel',                 preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 3,},
    RULEUTC_ProductionToggle = {    helpText = "toggle_production", bitmapId = 'production',            preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 4,},
    RULEUTC_StealthToggle = {       helpText = "toggle_stealth",    bitmapId = 'stealth',               preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 5,},
    RULEUTC_GenericToggle = {       helpText = "toggle_generic",    bitmapId = 'production',            preferredSlot = 10, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 6,},
    RULEUTC_SpecialToggle = {       helpText = "toggle_special",    bitmapId = 'activate-weapon',       preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 7,},
    RULEUTC_CloakToggle = {         helpText = "toggle_cloak",      bitmapId = 'intel-counter',         preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 8,},
}

local specialOrdersTable = {
    RULEUCC_Pause = {behavior = pauseFunc},
}


# this is a used as a set
local commonOrders = {
    RULEUCC_Move = true,
    RULEUCC_Attack = true,
    RULEUCC_Patrol = true,
    RULEUCC_Stop = true,
    RULEUCC_Guard = true, 
    RULEUCC_RetaliateToggle = true, 
}

--[[
Add an order to a particular slot, destroys what's currently in the slot if anything
Returns checkbox if you need to add any data to the structure

The orderInfo format is:
{
    helpText = <string>,    -- 
    bitmapId = <string>,    -- the id used to construct the bitmap name (see GetOrderBitmapNames above)
    disabled = <bool>,      -- if true, button will start disabled
    behavior = <function>,  -- function(self, modifiers) this is the checkbox OnClick behavior
}

Since this is a table, if you need any more information, for instance, the command to be emmited from the OnClick
you can add it to the table and it will be ignored, so you're safe to put whatever info you need in to it. When the
OnClick callback is called, self._data will contain this info.
--]]

local function AddOrder(orderInfo, slot, batchMode)
    batchMode = batchMode or false
    
    local checkbox = Checkbox(controls.orderButtonGrid, GetOrderBitmapNames(orderInfo.bitmapId))

    -- set the info in to the data member for retrieval
    checkbox._data = orderInfo
    
    -- set disabled state if needed
    if orderInfo.disabled then
        checkbox:Disable()
    end
    
    -- set up initial help text
    checkbox._curHelpText = orderInfo.helpText

    -- set up click handler
    checkbox.OnClick = orderInfo.behavior
    
    if orderInfo.onframe then
        checkbox.EnableEffect = Bitmap(checkbox, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
        LayoutHelpers.AtCenterIn(checkbox.EnableEffect, checkbox)
        checkbox.EnableEffect:DisableHitTest()
        checkbox.EnableEffect:SetAlpha(0)
        checkbox.EnableEffect.Incrimenting = false
        checkbox.EnableEffect.OnFrame = function(self, deltatime)
            local alpha
            if self.Incrimenting then
                alpha = self.Alpha + (deltatime * 2)
                if alpha > 1 then
                    alpha = 1
                    self.Incrimenting = false
                end
            else
                alpha = self.Alpha - (deltatime * 2)
                if alpha < 0 then
                    alpha = 0
                    self.Incrimenting = true
                end
            end
            self.Height:Set(function() return checkbox.Height() + (checkbox.Height() * alpha*.5) end)
            self.Width:Set(function() return checkbox.Height() + (checkbox.Height() * alpha*.5) end)
            self.Alpha = alpha
            self:SetAlpha(alpha*.45)
        end
        checkbox:SetNeedsFrameUpdate(true)
        checkbox.OnFrame = orderInfo.onframe
        checkbox.OnEnable = function(self)
            self.EnableEffect:SetNeedsFrameUpdate(true)
            self.EnableEffect.Incrimenting = false
            self.EnableEffect:SetAlpha(1)
            self.EnableEffect.Alpha = 1
            Checkbox.OnEnable(self)
        end
        checkbox.OnDisable = function(self)
            self.EnableEffect:SetNeedsFrameUpdate(false)
            self.EnableEffect:SetAlpha(0)
            Checkbox.OnDisable(self)
        end
    end

    # set up tooltips
    checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
            end                
            CreateMouseoverDisplay(self, self._curHelpText, 1)
            glowThread = CreateOrderGlow(self)
        elseif event.Type == 'MouseExit' then
            if controls.mouseoverDisplay then
                controls.mouseoverDisplay:Destroy()
                controls.mouseoverDisplay = false
            end
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
                KillThread(glowThread)
            end
        end
        Checkbox.HandleEvent(self, event)
    end

    # calculate row and column, remove old item, add new checkbox
    local cols, rows = controls.orderButtonGrid:GetDimensions()
    local row = math.ceil(slot / cols)
    local col = math.mod(slot - 1, cols) + 1
    controls.orderButtonGrid:DestroyItem(col, row, batchMode)
    controls.orderButtonGrid:SetItem(checkbox, col, row, batchMode)    

    return checkbox
end

# creates the buttons for the common orders, and then disables them if they aren't in the order set
local function CreateCommonOrders(availableOrders)
    for key in commonOrders do
        local orderInfo = standardOrdersTable[key]
        orderInfo.disabled = true

        local orderCheckbox = AddOrder(orderInfo, orderInfo.preferredSlot, true)
        orderCheckbox._order = key
        orderCheckbox._toggleState = 0

        if orderInfo.initialStateFunc then
            orderInfo.initialStateFunc(orderCheckbox, currentSelection)
        end
      
        orderCheckboxMap[key] = orderCheckbox
    end
    
    for index, availOrder in availableOrders do
        if not standardOrdersTable[availOrder] then continue end   # skip any orders we don't have in our table
        if commonOrders[availOrder] then
            local ck = orderCheckboxMap[availOrder]
            ck:Enable()
        end
    end
end

# creates the buttons for the alt orders, placing them as possible
local function CreateAltOrders(availableOrders, availableToggles, units)
#TODO? it would indeed be easier if the alt orders slot was in the blueprint, but for now try
#to determine where they go by using preferred slots

    --Look for units in the selection that have special ability buttons
    --If any are found, add the ability information to the standard order table
    if units and categories.ABILITYBUTTON and EntityCategoryFilterDown(categories.ABILITYBUTTON, units) then
        for index, unit in units do
            local tempBP = UnitData[unit:GetEntityId()]
            if tempBP.Abilities then
                for abilityIndex, ability in tempBP.Abilities do
                    table.insert(availableOrders, abilityIndex)
                    standardOrdersTable[abilityIndex] = table.merged(ability, import('/lua/abilitydefinition.lua').abilities[abilityIndex])
                    standardOrdersTable[abilityIndex].behavior = AbilityButtonBehavior
                end
            end
        end
    end
    local asssitingUnitList = {}
    if EntityCategoryFilterDown(categories.PODSTAGINGPLATFORM, units) or EntityCategoryFilterDown(categories.POD, units) then
        local PodStagingPlatforms = EntityCategoryFilterDown(categories.PODSTAGINGPLATFORM, units)
        if table.getn(PodStagingPlatforms) == 0 then
            PodStagingPlatforms = EntityCategoryFilterDown(categories.POD, units)
        end
        local assistingUnits = GetAssistingUnitsList(PodStagingPlatforms)
        if assistingUnits[1] then
            table.insert(availableOrders, 'DroneL')
            asssitingUnitList['DroneL'] = assistingUnits[1]
        end
        if assistingUnits[2] then
            table.insert(availableOrders, 'DroneR')
            asssitingUnitList['DroneR'] = assistingUnits[2]
        end
    end
    
    # determine what slots to put alt orders
    # we first want a table of slots we want to fill, and what orders want to fill them
    local desiredSlot = {}
    for index, availOrder in availableOrders do
        if standardOrdersTable[availOrder] then 
            local preferredSlot = standardOrdersTable[availOrder].preferredSlot
            if not desiredSlot[preferredSlot] then
                desiredSlot[preferredSlot] = {}
            end
            table.insert(desiredSlot[preferredSlot], availOrder)
        else
            if specialOrdersTable[availOrder] != nil then
                specialOrdersTable[availOrder].behavior()
            end
        end
    end

    for index, availToggle in availableToggles do
        if standardOrdersTable[availToggle] then 
            local preferredSlot = standardOrdersTable[availToggle].preferredSlot
            if not desiredSlot[preferredSlot] then
                desiredSlot[preferredSlot] = {}
            end
            table.insert(desiredSlot[preferredSlot], availToggle)
        else
            if specialOrdersTable[availToggle] != nil then
                specialOrdersTable[availToggle].behavior()
            end
        end
    end

    # now go through that table and determine what doesn't fit and look for slots that are empty
    # since this is only alt orders, just deal with slots 7-12
    local orderInSlot = {}
    
    # go through first time and add all the first entries to their preferred slot
    for slot = firstAltSlot,numSlots do
        if desiredSlot[slot] then
            orderInSlot[slot] = desiredSlot[slot][1]
        end
    end

    # now put any additional entries wherever they will fit
    for slot = firstAltSlot,numSlots do
        if desiredSlot[slot] and table.getn(desiredSlot[slot]) > 1 then
            for index, item in desiredSlot[slot] do
                if index > 1 then
                    local foundFreeSlot = false
                    for newSlot = firstAltSlot, numSlots do
                        if not orderInSlot[newSlot] then
                            orderInSlot[newSlot] = item
                            foundFreeSlot = true
                            break
                        end
                    end
                    if not foundFreeSlot then
                        WARN("No free slot for order: " .. item)
                        # could break here, but don't, then you'll know how many extra orders you have
                    end
                end
            end
        end
    end

    # now map it the other direction so it's order to slot
    local slotForOrder = {}
    for slot, order in orderInSlot do
        slotForOrder[order] = slot
    end
    --LOG(repr(availableOrders), repr(orderInSlot), repr(slotForOrder))
    
    # create the alt order buttons
    for index, availOrder in availableOrders do
        if not standardOrdersTable[availOrder] then continue end   # skip any orders we don't have in our table
        if not commonOrders[availOrder] then
            local orderInfo = standardOrdersTable[availOrder] or AbilityInformation[availOrder]
            orderInfo.disabled = false
            local orderCheckbox = AddOrder(orderInfo, slotForOrder[availOrder], true)

            orderCheckbox._order = availOrder
            
            if standardOrdersTable[availOrder].script then
                orderCheckbox._script = standardOrdersTable[availOrder].script
            end
            
            if asssitingUnitList[availOrder] then
                orderCheckbox._unit = asssitingUnitList[availOrder]
            end

            if orderInfo.initialStateFunc then
                orderInfo.initialStateFunc(orderCheckbox, currentSelection)
            end

            orderCheckboxMap[availOrder] = orderCheckbox
        end
    end

    for index, availToggle in availableToggles do
        if not standardOrdersTable[availToggle] then continue end   # skip any orders we don't have in our table
        if not commonOrders[availToggle] then
            local orderInfo = standardOrdersTable[availToggle] or AbilityInformation[availToggle]
            orderInfo.disabled = false
            local orderCheckbox = AddOrder(orderInfo, slotForOrder[availToggle], true)

            orderCheckbox._order = availToggle
            
            if standardOrdersTable[availToggle].script then
                orderCheckbox._script = standardOrdersTable[availToggle].script
            end
            
            if asssitingUnitList[availToggle] then
                orderCheckbox._unit = asssitingUnitList[availToggle]
            end

            if orderInfo.initialStateFunc then
                orderInfo.initialStateFunc(orderCheckbox, currentSelection)
            end

            orderCheckboxMap[availToggle] = orderCheckbox
        end
    end
end

-- called by gamemain when new orders are available, 
function SetAvailableOrders(availableOrders, availableToggles, newSelection)
    # save new selection
    currentSelection = newSelection
    
    # clear existing orders
    orderCheckboxMap = {}
    controls.orderButtonGrid:DestroyAllItems(true)
    
    CreateCommonOrders(availableOrders)
    
    local numValidOrders = 0
    for i, v in availableOrders do
        if standardOrdersTable[v] then
            numValidOrders = numValidOrders + 1
        end
    end
    for i, v in availableToggles do
        if standardOrdersTable[v] then
            numValidOrders = numValidOrders + 1
        end
    end
    
    if numValidOrders <= 12 then
        CreateAltOrders(availableOrders, availableToggles, currentSelection)
    end
    
    controls.orderButtonGrid:EndBatch()
    if numValidOrders == 0 and controls.bg.Mini then
        controls.bg.Mini(true)
    elseif controls.bg.Mini then
        controls.bg.Mini(false)
    end
end

function CreateControls()
    if controls.mouseoverDisplay then
        controls.mouseoverDisplay:Destroy()
        controls.mouseoverDisplay = false
    end
    if not controls.bg then
        controls.bg = Bitmap(controls.controlClusterGroup)
    end
    if not controls.orderButtonGrid then
        CreateOrderButtonGrid()
    end
end

function SetLayout(layout)
    layoutVar = layout

    # clear existing orders
    orderCheckboxMap = {}
    if controls and controls.orderButtonGrid then
        controls.orderButtonGrid:DeleteAndDestroyAll(true)
    end

    CreateControls()
    import(UIUtil.GetLayoutFilename('orders')).SetLayout()

    # created greyed out orders on setup
    CreateCommonOrders({})

    controls.orderButtonGrid:EndBatch()
end

-- called from gamemain to create control
function SetupOrdersControl(parent, mfd)
    controls.controlClusterGroup = parent
    controls.mfdControl = mfd

    SetLayout(UIUtil.currentLayout)

    -- setup command mode behaviors
    import('/lua/ui/game/commandmode.lua').AddStartBehavior(
        function(commandMode, data)
            local orderCheckbox = orderCheckboxMap[data]
            if orderCheckbox then
                orderCheckbox:SetCheck(true)
            end
        end
    )
    import('/lua/ui/game/commandmode.lua').AddEndBehavior(
        function(commandMode, data)
            local orderCheckbox = orderCheckboxMap[data]
            if orderCheckbox then
                orderCheckbox:SetCheck(false)
            end
        end
    )
    
    return controls.bg
end






















--()((()-- only these functions should be necessary, but local stuff fucks up
local function CreateOrderButtonGrid()
    controls.orderButtonGrid = Grid(controls.bg, 47, 47)
    controls.orderButtonGrid:SetName("Orders Grid")
	controls.orderButtonGrid:DeleteAll()
end

local function GetOrderBitmapNames(bitmapId)
    if bitmapId == nil then
        LOG("Error - nil bitmap passed to GetOrderBitmapNames")
        bitmapId = "basic-empty"    --TODO do I really want to default it?
    end
    
    local button_prefix = "/mods/OllosInterface/textures/icons_orders/" .. bitmapId .. "_btn_"
    return button_prefix .. "up.dds"
        ,  button_prefix .. "up_sel.dds"
        ,  button_prefix .. "over.dds"
        ,  button_prefix .. "over_sel.dds"
        ,  button_prefix .. "dis.dds"
        ,  button_prefix .. "dis_sel.dds"
        , "UI_Action_MouseDown", "UI_Action_Rollover"   -- sets click and rollover cues
end